@@ -1,15 +1,20 @@
 
 //              Copyright Catch2 Authors
 // Distributed under the Boost Software License, Version 1.0.
-//   (See accompanying file LICENSE_1_0.txt or copy at
+//   (See accompanying file LICENSE.txt or copy at
 //        https://www.boost.org/LICENSE_1_0.txt)
 
 // SPDX-License-Identifier: BSL-1.0
+
 #include <catch2/internal/catch_debug_console.hpp>
+
 #include <catch2/internal/catch_config_android_logwrite.hpp>
-#include <catch2/internal/catch_stream.hpp>
 #include <catch2/internal/catch_platform.hpp>
 #include <catch2/internal/catch_windows_h_proxy.hpp>
+#include <catch2/catch_user_config.hpp>
+#include <catch2/internal/catch_stdstreams.hpp>
+
+#include <ostream>
 
 #if defined(CATCH_CONFIG_ANDROID_LOGWRITE)
 #include <android/log.h>
